/*******************************************************************************
* File Name: CAN_RX.c
* Version 1.50
*
* Description:
*  The CAN Component provide functionality of Control Area Network.
*  Two types of mailbox configurations available "Full" and "Basic".
*
* Note:
*  CAN configuration puts as constant to ROM and could change only directly
*  registers write by user.
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "CAN_RX.h"

uint8 CAN_RX_initVar = 0u;

/* Iniital values of CAN RX and TX registers */
const CAN_RX_RX_CFG CYCODE CAN_RX_RXConfigStruct[] =
{
    { 0u, 0x28u, 0x1FFFF9u, 0xA00000u },
    { 1u, 0x0u, 0x0u, 0x0u },
    { 2u, 0x0u, 0x0u, 0x0u },
    { 3u, 0x0u, 0x0u, 0x0u },
    { 4u, 0x0u, 0x0u, 0x0u },
    { 5u, 0x0u, 0x0u, 0x0u },
    { 6u, 0x0u, 0x0u, 0x0u },
    { 7u, 0x0u, 0x0u, 0x0u },
    { 8u, 0x0u, 0x0u, 0x0u },
    { 9u, 0x0u, 0x0u, 0x0u },
    { 10u, 0x0u, 0x0u, 0x0u },
    { 11u, 0x0u, 0x0u, 0x0u },
    { 12u, 0x0u, 0x0u, 0x0u },
    { 13u, 0x0u, 0x0u, 0x0u },
    { 14u, 0x0u, 0x0u, 0x0u },
    { 15u, 0x0u, 0x0u, 0x0u }
};

const CAN_RX_TX_CFG CYCODE CAN_RX_TXConfigStruct[] =
{
    { 0u, 0x880008u, 0x0u },
    { 1u, 0x880008u, 0x0u },
    { 2u, 0x880008u, 0x0u },
    { 3u, 0x880008u, 0x0u },
    { 4u, 0x880008u, 0x0u },
    { 5u, 0x880008u, 0x0u },
    { 6u, 0x880008u, 0x0u },
    { 7u, 0x880008u, 0x0u }
};


/*******************************************************************************
* FUNCTION NAME:   CAN_RX_Init
********************************************************************************
*
* Summary:
*  Inits/Restores default CAN configuration provided with customizer.
*
* Parameters:
*  None.
*
* Return:
*  Indication whether the configuration has been accepted or rejected.
*
* Side Effects:
*  All registers will be reset to their initial values. This will re-initialize 
*  the component with the following exceptions. It will not clear data from 
*  the mailboxes.
*  Enable power to the CAN Core.
*
*******************************************************************************/
uint8 CAN_RX_Init(void) 
{
    uint8 i;
    uint8 result = CAN_RX_FAIL;
    uint8 localResult = CYRET_SUCCESS;    
    
    /* Disable Interrupt. */
    CyIntDisable(CAN_RX_ISR_NUMBER);

    /* Set the ISR to point to the CAN_RX_ISR Interrupt. */
    CyIntSetVector(CAN_RX_ISR_NUMBER, CAN_RX_ISR);

    /* Set the priority. */
    CyIntSetPriority(CAN_RX_ISR_NUMBER, CAN_RX_ISR_PRIORITY);    

    /* Enable CAN block in Active mode */
    CAN_RX_PM_ACT_CFG_REG |= CAN_RX_ACT_PWR_EN;
    
    /* Enable CAN block in Alternate Active (Standby) mode */
    CAN_RX_PM_STBY_CFG_REG |= CAN_RX_STBY_PWR_EN;    

    if (CAN_RX_SetPreScaler(CAN_RX_BITRATE) == CYRET_SUCCESS)
    {
        if (CAN_RX_SetArbiter(CAN_RX_ARBITER) == CYRET_SUCCESS)
        {
            if (CAN_RX_SetTsegSample(CAN_RX_CFG_REG_TSEG1, CAN_RX_CFG_REG_TSEG2,
                CAN_RX_CFG_REG_SJW, CAN_RX_SAMPLING_MODE) == CYRET_SUCCESS)
            {
                if (CAN_RX_SetRestartType(CAN_RX_RESET_TYPE) == CYRET_SUCCESS)
                {
                    if (CAN_RX_SetEdgeMode(CAN_RX_SYNC_EDGE) == CYRET_SUCCESS)
                    {
                        /* Initialize TX maiboxes */
                        for(i = 0u; i < CAN_RX_NUMBER_OF_TX_MAILBOXES; i++)
                        {
                            if (CAN_RX_TxBufConfig((CAN_RX_TX_CFG *)
                                (&CAN_RX_TXConfigStruct[i])) != CYRET_SUCCESS)
                            {
                                localResult = CAN_RX_FAIL;
                                break;
                            }
                        }

                        if (localResult == CYRET_SUCCESS)
                        {
                            /* Initialize RX mailboxes */
                            for(i = 0u; i < CAN_RX_NUMBER_OF_RX_MAILBOXES; i++)
                            {
                                if (CAN_RX_RxBufConfig((CAN_RX_RX_CFG *)
                                    (&CAN_RX_RXConfigStruct[i])) != CYRET_SUCCESS)
                                {
                                    localResult = CAN_RX_FAIL;
                                    break;
                                }
                            }

                            if (localResult == CYRET_SUCCESS)
                            {
                                /* Write IRQ Mask */
                                if (CAN_RX_SetIrqMask(CAN_RX_INIT_INTERRUPT_MASK) ==
                                    CYRET_SUCCESS)
                                {
                                    /* Active mode always */
                                    if (CAN_RX_SetOpMode(CAN_RX_ACTIVE_MODE) == CYRET_SUCCESS)
                                    {
                                        result = CYRET_SUCCESS;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }    

    return(result);
}


/*******************************************************************************
* Function Name: CAN_RX_Enable
********************************************************************************
*
* Summary:   
*  This function enable CAN Component and ISR of CAN Component. 
*
* Parameters:  
*  None.
*
* Return: 
*  Indication whether register is written and verified..
*
*******************************************************************************/
uint8 CAN_RX_Enable(void) 
{
    uint8 result = CAN_RX_FAIL;
    uint8 enableInterrupts = 0u;
    
    enableInterrupts = CyEnterCriticalSection();
    
    /* Enable CAN block in Active mode */
    CAN_RX_PM_ACT_CFG_REG |= CAN_RX_ACT_PWR_EN;

    /* Enable CAN block in Alternate Active (Standby) mode */
    CAN_RX_PM_STBY_CFG_REG |= CAN_RX_STBY_PWR_EN;
    
    CyExitCriticalSection(enableInterrupts);
    
    /* Enable isr */
    CyIntEnable(CAN_RX_ISR_NUMBER);
    
    /* Sets the CAN controller to run mode */
    CAN_RX_CMD_REG.byte[0u] |= CAN_RX_MODE_MASK;
    
    /* Verify that bit is set */
    if ((CAN_RX_CMD_REG.byte[0u] & CAN_RX_MODE_MASK) == CAN_RX_MODE_MASK)
    {
        result = CYRET_SUCCESS;
    }
    
    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   CAN_RX_Start
********************************************************************************
*
* Summary:
*  This function sets CAN Component into Run mode. Starts the Rate Counter if
*  polling mailboxes available.
*
* Parameters:
*  None. 
*
* Return:
*  Indication whether register is written and verified.  
*
* Global variables:
*  CAN_RX_initVar - used to check initial configuration, modified on
*  first function call.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 CAN_RX_Start(void)
{
    uint8 result = CAN_RX_FAIL;
    
    if (CAN_RX_initVar == 0u)
    {
        result = CAN_RX_Init();
        CAN_RX_initVar = 1u;        
    }
    
    if (result == CYRET_SUCCESS)
    {
        result = CAN_RX_Enable();
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   CAN_RX_Stop
********************************************************************************
*
* Summary:
*  This function sets CAN Component into Stop mode.
*
* Parameters:
*  None.
*
* Return:
*  Indication whether register is written and verified.
*
* Side Effects:
*  Disable power to the CAN Core.
*
*******************************************************************************/
uint8 CAN_RX_Stop(void) 
{
    uint8 result = CAN_RX_FAIL;
    uint8 enableInterrupts = 0u;        
    
    /* Sets the CAN controller to stop mode */
    CAN_RX_CMD_REG.byte[0u] &= ~CAN_RX_MODE_MASK;

    /* Verify that bit is cleared */
    if ((CAN_RX_CMD_REG.byte[0u] & CAN_RX_MODE_MASK) == 0u)
    {
        result = CYRET_SUCCESS;
    }
    
    /* Disable interrupt */
    CyIntDisable(CAN_RX_ISR_NUMBER); 
    
    enableInterrupts = CyEnterCriticalSection(); 

    /* Disable CAN block in Active mode */
    CAN_RX_PM_ACT_CFG_REG &= ~CAN_RX_ACT_PWR_EN;

    /* Disable CAN block in Alternate Active (Standby) mode template */
    CAN_RX_PM_STBY_CFG_REG &= ~CAN_RX_STBY_PWR_EN;
    
    CyExitCriticalSection(enableInterrupts);

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   CAN_RX_GlobalIntEnable
********************************************************************************
*
* Summary:
*  This function enables Global Interrupts from CAN Core.
*
* Parameters:
*  None.
*
* Return:
*  Indication whether register is written and verified.
*
*******************************************************************************/
uint8 CAN_RX_GlobalIntEnable(void) 
{
    uint8 result = CAN_RX_FAIL;

    CAN_RX_INT_EN_REG.byte[0u] |= CAN_RX_GLOBAL_INT_MASK;

    /* Verify that bit is set */
    if ((CAN_RX_INT_EN_REG.byte[0u] & CAN_RX_GLOBAL_INT_MASK) == CAN_RX_GLOBAL_INT_MASK)
    {
        result = CYRET_SUCCESS;
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   CAN_RX_GlobalIntDisable
********************************************************************************
*
* Summary:
*  This function disables Global Interrupts from CAN Core.
*
* Parameters:
*  None.
*
* Return:
*  Indication whether register is written and verified.
*
*******************************************************************************/
uint8 CAN_RX_GlobalIntDisable(void) 
{
    uint8 result = CAN_RX_FAIL;

    CAN_RX_INT_EN_REG.byte[0u] &= ~CAN_RX_GLOBAL_INT_MASK;

    /* Verify that bit is cleared */
    if ((CAN_RX_INT_EN_REG.byte[0u] & CAN_RX_GLOBAL_INT_MASK) == 0u)
    {
        result = CYRET_SUCCESS;
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   CAN_RX_SetPreScaler
********************************************************************************
*
* Summary:
*  This function sets PreScaler for generation the time quantum which defines
*  the time quanta. Value between 0x0 and 0x7FFF are valid.
*
* Parameters:
*  bitrate: PreScaler value.
*
* Return:
*  Indication whether register is written and verified.
*
*******************************************************************************/
uint8 CAN_RX_SetPreScaler(uint16 bitrate) 
{
    uint8 result = CAN_RX_OUT_OF_RANGE;

    if (bitrate <= CAN_RX_BITRATE_MASK)
    {
        result = CAN_RX_FAIL;

        /* Set prescaler */
        CY_SET_REG16((reg16 *) &CAN_RX_CFG_REG.byte[2u], bitrate);

        /* Verify that prescaler is set */
        if (CY_GET_REG16((reg16 *) &CAN_RX_CFG_REG.byte[2u]) == bitrate)
        {
            result = CYRET_SUCCESS;
        }
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   CAN_RX_SetArbiter
********************************************************************************
*
* Summary:
*  This function sets arbitration type for transmit mailboxes. Types of
*  arbiters are Round Robin and Fixed priority. Value 0 and 1 are valid.
*
* Parameters:
*  arbiter: Arbitaration type for transmit mailboxes.
*
* Return:
*  Indication whether register is written and verified.
*
*******************************************************************************/
uint8 CAN_RX_SetArbiter(uint8 arbiter) 
{
    uint8 result = CAN_RX_FAIL;

    if (arbiter == CAN_RX_ROUND_ROBIN)
    {
        CAN_RX_CFG_REG.byte[1u] &= ~CAN_RX_ARBITRATION_MASK;    /* Round Robin */

        /* Verify that bit is cleared */
        if ((CAN_RX_CFG_REG.byte[1u] & CAN_RX_ARBITRATION_MASK) == 0u)
        {
            result = CYRET_SUCCESS;
        }        
    }
    else
    {
        CAN_RX_CFG_REG.byte[1u] |= CAN_RX_ARBITRATION_MASK;    /* Fixed priority */

        /* Verify that bit is set */
        if ((CAN_RX_CFG_REG.byte[1u] & CAN_RX_ARBITRATION_MASK) ==
            CAN_RX_ARBITRATION_MASK)
        {
            result = CYRET_SUCCESS;
        }
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME: CAN_RX_SetTsegSample
********************************************************************************
*
* Summary:
*  This function configures: Time segment 1, Time segment 2, Sampling Mode
*  and Synchronization Jump Width.
*
* Parameters:
*  cfgTseg1: Time segment 1, value between 0x2 and 0xF are valid;
*  cfgTseg2: Time segment 2, value between 0x1 and 0x7 are valid;
*  sjw: Synchronization Jump Width, value between 0x0 and 0x3 are valid;
*  sm: Sampling Mode, one or three sampling points are used.
*
* Return:
*  Indication whether register is written and verified.
*
*******************************************************************************/
uint8 CAN_RX_SetTsegSample(uint8 cfgTseg1, uint8 cfgTseg2, uint8 sjw, uint8 sm)
                                     
{
    uint8 result = CAN_RX_OUT_OF_RANGE;
    uint8 cfgTemp;

    if ((cfgTseg1 >= CAN_RX_CFG_REG_TSEG1_LOWER_LIMIT) && (cfgTseg1 <=
         CAN_RX_CFG_REG_TSEG1_UPPER_LIMIT))
    {
        if (((cfgTseg2 >= CAN_RX_CFG_REG_TSEG2_LOWER_LIMIT) &&
             (cfgTseg2 <= CAN_RX_CFG_REG_TSEG2_UPPER_LIMIT)) || ((sm == CAN_RX_ONE_SAMPLE_POINT) &&
             (cfgTseg2 == CAN_RX_CFG_REG_TSEG2_EXCEPTION)))
        {
            if ((sjw <= CAN_RX_CFG_REG_SJW_LOWER_LIMIT) && (sjw <= cfgTseg1) && (sjw <= cfgTseg2))
            {
                result = CAN_RX_FAIL;

                cfgTemp = CAN_RX_CFG_REG.byte[1];
                cfgTemp &= ~CAN_RX_CFG_REG_TSEG1_MASK;
                cfgTemp |= cfgTseg1;

                /* write register byte 1 */
                CAN_RX_CFG_REG.byte[1u] = cfgTemp;

                /* Verify 2nd byte of CAN_RX_CFG_REG register */
                if (CAN_RX_CFG_REG.byte[1u] == cfgTemp)
                {
                    /* read and clean bits */
                    cfgTemp = CAN_RX_CFG_REG.byte[0u];
                    cfgTemp &= ~(CAN_RX_CFG_REG_TSEG2_MASK | CAN_RX_CFG_REG_SJW_MASK |
                                  CAN_RX_SAMPLE_MODE_MASK);

                    cfgTemp = 0u;
                    /* Set appropriate bits */
                    if (sm != CAN_RX_ONE_SAMPLE_POINT)
                    {
                        cfgTemp |= CAN_RX_SAMPLE_MODE_MASK;
                    }
                    cfgTemp |= ((cfgTseg2 << CAN_RX_CFG_REG_TSEG2_SHIFT) |
                                 (sjw << CAN_RX_CFG_REG_SJW_SHIFT));

                    /* write register byte 0 */
                    CAN_RX_CFG_REG.byte[0u] = cfgTemp;

                    /* Verify 1st byte of CAN_RX_CFG_REG register */
                    if (CAN_RX_CFG_REG.byte[0u] == cfgTemp)
                    {
                        result = CYRET_SUCCESS;
                    }
                }
            }
        }
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   CAN_RX_SetRestartType
********************************************************************************
*
* Summary:
*  This function sets Reset type. Types of Reset are Automatic and Manual.
*  Manual Reset is recommended setting. Value 0 and 1 are valid.
*
* Parameters:
*  reset: Reset Type.
*
* Return:
*  Indication whether register is written and verified.
*
*******************************************************************************/
uint8 CAN_RX_SetRestartType(uint8 reset) 
{
    uint8 result = CAN_RX_FAIL;

    if (reset == CAN_RX_RESTART_BY_HAND)
    {
        CAN_RX_CFG_REG.byte[0u] &= ~CAN_RX_RESET_MASK;   /* Manual reset */

        /* Verify that bit is cleared */
        if ((CAN_RX_CFG_REG.byte[0u] & CAN_RX_RESET_MASK) == 0u)
        {
            result = CYRET_SUCCESS;
        }
    }
    else
    {
        CAN_RX_CFG_REG.byte[0u] |= CAN_RX_RESET_MASK;    /* Automatic restart */

        /* Verify that bit is set */
        if ((CAN_RX_CFG_REG.byte[0u] & CAN_RX_RESET_MASK) == CAN_RX_RESET_MASK)
        {
            result = CYRET_SUCCESS;
        }
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   CAN_RX_SetEdgeMode
********************************************************************************
*
* Summary:
*  This function sets Edge Mode. Modes are 'R' to 'D'(Reccessive to Dominant)
*  and Both edges are used. Value 0 and 1 are valid.
*
* Parameters:
*  edge: Edge Mode.
*
* Return:
*  Indication whether register is written and verified.
*
*******************************************************************************/
uint8 CAN_RX_SetEdgeMode(uint8 edge) 
{
    uint8 result = CAN_RX_FAIL;

    if (edge == CAN_RX_EDGE_R_TO_D)
    {
        /* Recessive to Dominant is used for synchronization */
        CAN_RX_CFG_REG.byte[0u] &= ~CAN_RX_EDGE_MODE_MASK;

        /* Verify that bit is cleared */
        if ((CAN_RX_CFG_REG.byte[0u] & CAN_RX_EDGE_MODE_MASK) == 0u)
        {
            result = CYRET_SUCCESS;
        }                      
    }
    else
    {
        /* Both edges are to be used */
        CAN_RX_CFG_REG.byte[0u] |= CAN_RX_EDGE_MODE_MASK;

        /* Verify that bit is set */
        if ((CAN_RX_CFG_REG.byte[0u] & CAN_RX_EDGE_MODE_MASK) == CAN_RX_EDGE_MODE_MASK)
        {
            result = CYRET_SUCCESS;
        }
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   CAN_RX_SetOpMode
********************************************************************************
*
* Summary:
*  This function sets Operation Mode. Operations Modes are Active of Listen
*  Only. Value 0 and 1 are valid.
*
* Parameters:
*  opMode: Operation Mode value.
*
* Return:
*  Indication whether register is written and verified.
*
*******************************************************************************/
uint8 CAN_RX_SetOpMode(uint8 opMode) 
{
    uint8 result = CAN_RX_FAIL;

    if (opMode == CAN_RX_ACTIVE_MODE)
    {
        /* CAN in Active mode */
        CAN_RX_CMD_REG.byte[0u] &= ~CAN_RX_OPMODE_MASK;

        /* Verify that bit is cleared */
        if ((CAN_RX_CMD_REG.byte[0u] & CAN_RX_OPMODE_MASK) == 0u)
        {
            result = CYRET_SUCCESS;
        }                
    }
    else
    {
        /* CAN listen only */
        CAN_RX_CMD_REG.byte[0u] |= CAN_RX_OPMODE_MASK;

        /* Verify that bit is set */
        if ((CAN_RX_CMD_REG.byte[0u] & CAN_RX_OPMODE_MASK) == CAN_RX_OPMODE_MASK)
        {
            result = CYRET_SUCCESS;
        }
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   CAN_RX_RXRegisterInit
********************************************************************************
*
* Summary:
*  This function writes only receive CAN registers.
*
* Parameters:
*  regAddr: Pointer to CAN receive register;
*  config:  Value that will be written in register.
*
* Return:
*  Indication whether register is written and verified.
*
*******************************************************************************/
uint8 CAN_RX_RXRegisterInit(reg32 *regAddr, uint32 config)
                                      
{
    uint8 result = CAN_RX_OUT_OF_RANGE;    
    
    if ( (((uint32) regAddr & 0x0000FFFFu) >= ((uint32)CAN_RX_RX_FIRST_REGISTER_PTR & 0x0000FFFFu)) &&
         ((((uint32) regAddr & 0x0000FFFFu)) <= ((uint32)CAN_RX_RX_LAST_REGISTER_PTR & 0x0000FFFFu)) )
    {
        result = CAN_RX_FAIL;
        
        if ((((uint32) regAddr & 0x0000FFFFu) % CAN_RX_RX_CMD_REG_WIDTH) == 0u)
        {
            config |= CAN_RX_RX_WPN_SET;

            /* Disable Interrupt. */
            CyIntDisable(CAN_RX_ISR_NUMBER);
        
            /* Write defined RX CMD registers */
            CY_SET_REG32(regAddr, config);
            
            /* Enable isr */
            CyIntEnable(CAN_RX_ISR_NUMBER);


            /* Verify register */
            if ( (CY_GET_REG32(regAddr) & CAN_RX_RX_READ_BACK_MASK) ==
                (config & CAN_RX_RX_READ_BACK_MASK) )
            {
                result = CYRET_SUCCESS;
            }
        }
        /* All registers except RX CMD*/
        else
        {            
            /* Disable Interrupt. */
            CyIntDisable(CAN_RX_ISR_NUMBER);
            
            /* Write defined CAN receive register */
            CY_SET_REG32(regAddr, config);
            
            /* Enable isr */
            CyIntEnable(CAN_RX_ISR_NUMBER);

            /* Verify register */
            if (CY_GET_REG32(regAddr) == config)
            {
                result = CYRET_SUCCESS;
            }
        }
    }
    
    return (result);
}


/*******************************************************************************
* FUNCTION NAME:   CAN_RX_SetIrqMask
********************************************************************************
*
* Summary:
*  This function sets to enable/disable particular interrupt sources. Interrupt
*  Mask directly write to CAN Interrupt Enable register.
*
* Parameters:
*  mask: Interrupt enable/disable request. 1 bit per interrupt source.
*
* Return:
*  Indication whether register is written and verified.
*
*******************************************************************************/
uint8 CAN_RX_SetIrqMask(uint16 mask) 
{
    uint8 result = CAN_RX_FAIL;

    /* Write byte 0 and 1 of CAN_RX_INT_EN_REG register */
    CY_SET_REG16((reg16 *) &CAN_RX_INT_EN_REG, mask);

    /* Verify CAN_RX_INT_EN_REG register */
    if (CY_GET_REG16((reg16 *) &CAN_RX_INT_EN_REG) == mask)
    {
        result = CYRET_SUCCESS;
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   CAN_RX_GetTXErrorflag
********************************************************************************
*
* Summary:
*  This function returns the bit that indicates if the number of TX errors
*  exceeds 0x60.
*
* Parameters:
*  None.
*
* Return:
*  Indication whether the number of TX errors exceeds 0x60.
*
*******************************************************************************/
uint8 CAN_RX_GetTXErrorFlag(void) 
{
    /* Get the state of the transmit error flag */
    return(((CAN_RX_ERR_SR_REG.byte[2u] & CAN_RX_TX_ERROR_FLAG_MASK) ==
             CAN_RX_TX_ERROR_FLAG_MASK) ? 1u : 0u);
}


/*******************************************************************************
* FUNCTION NAME:   CAN_RX_GetRXErrorFlag
********************************************************************************
*
* Summary:
*  This function returns the bit that indicates if the number of RX errors
*  exceeds 0x60.
*
* Parameters:
*  None.
*
* Return:
*  Indication whether the number of TX errors exceeds 0x60.
*
*******************************************************************************/
uint8 CAN_RX_GetRXErrorFlag(void) 
{
    /* Get the state of the receive error flag */
    return(((CAN_RX_ERR_SR_REG.byte[2u] & CAN_RX_RX_ERROR_FLAG_MASK) ==
             CAN_RX_RX_ERROR_FLAG_MASK) ? 1u : 0u);
}


/*******************************************************************************
* FUNCTION NAME:   CAN_RX_TXErrorCount
********************************************************************************
*
* Summary:
*  This function returns the number of Transmit Errors.
*
* Parameters:
*  None.
*
* Return:
*  Number of Transmit Errors.
*
*******************************************************************************/
uint8 CAN_RX_GetTXErrorCount(void) 
{
    /* Get the state of the transmit error count */
    return(CAN_RX_ERR_SR_REG.byte[0u]);    /* bits 7-0 */
}


/*******************************************************************************
* FUNCTION NAME:   CAN_RX_RXErrorCount
********************************************************************************
*
* Summary:
*  This function returns the number of Receive Errors.
*
* Parameters:
*  None.
*
* Return:
*  Number of Receive Errors.
*
*******************************************************************************/
uint8 CAN_RX_GetRXErrorCount(void) 
{
    /* Get the state of the receive error count */
    return(CAN_RX_ERR_SR_REG.byte[1u]);    /* bits 15-8 */
}


/*******************************************************************************
* FUNCTION NAME:   CAN_RX_GetErrorState
********************************************************************************
*
* Summary:
*  This function returns error status of CAN Component.
*
* Parameters:
*  None.
*
* Return:
*  Error status.
*
*******************************************************************************/
uint8 CAN_RX_GetErrorState(void) 
{
    /* Get the error state of the receiver */
    return(CAN_RX_ERR_SR_REG.byte[2u] & CAN_RX_ERROR_STATE_MASK);
}


/*******************************************************************************
* FUNCTION NAME:   CAN_RX_RxBufConfig
********************************************************************************
*
* Summary:
*  This function configures all receive registers for particular mailbox.
*
* Parameters:
*  rxConfig: Pointer to structure that contain all required values to configure
*  all receive registers for particular mailbox.
*
* Return:
*  Indication if particular configuration has been accepted or rejected.
*
*******************************************************************************/
uint8 CAN_RX_RxBufConfig(CAN_RX_RX_CFG *rxConfig) 
{
    uint8 result = CAN_RX_FAIL;

    /* Write RX CMD Register */
    CY_SET_REG32((reg32 *) &CAN_RX_RX[rxConfig->rxmailbox].rxcmd, (rxConfig->rxcmd |
                 CAN_RX_RX_WPN_SET));
    if ((CY_GET_REG32((reg32 *) & CAN_RX_RX[rxConfig->rxmailbox].rxcmd) &
         CAN_RX_RX_READ_BACK_MASK) == (rxConfig->rxcmd & CAN_RX_RX_WPN_CLEAR))
    {
        /* Write RX AMR Register */
        CY_SET_REG32((reg32 *) &CAN_RX_RX[rxConfig->rxmailbox].rxamr, rxConfig->rxamr);
        if (CY_GET_REG32((reg32 *) &CAN_RX_RX[rxConfig->rxmailbox].rxamr) == rxConfig->rxamr)
        {
            /* Write RX ACR Register */
            CY_SET_REG32((reg32 *) &CAN_RX_RX[rxConfig->rxmailbox].rxacr, rxConfig->rxacr);
            if (CY_GET_REG32((reg32 *) &CAN_RX_RX[rxConfig->rxmailbox].rxacr) == rxConfig->rxacr)
            {
                /* Write RX AMRD Register */
                CY_SET_REG32((reg32 *) &CAN_RX_RX[rxConfig->rxmailbox].rxamrd, 0xFFFFFFFFu);
                if (CY_GET_REG32((reg32 *) &CAN_RX_RX[rxConfig->rxmailbox].rxamrd) == 0xFFFFFFFFu)
                {
                    /* Write RX ACRD Register */
                    CY_SET_REG32((reg32 *) &CAN_RX_RX[rxConfig->rxmailbox].rxacrd, 0x00000000u);
                    if (CY_GET_REG32((reg32 *) &CAN_RX_RX[rxConfig->rxmailbox].rxacrd) == 0x00000000u)
                    {
                            result = CYRET_SUCCESS;
                    }
                }
            }
        }
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   CAN_RX_TxBufConfig
********************************************************************************
*
* Summary:
*  This function configures all transmit registers for particular mailbox.
*  Mailbox number contains CAN_RX_TX_CFG structure.
*
* Parameters:
*  txConfig: Pointer to structure that contain all required values to configure
*  all transmit registers for particular mailbox.
*
* Return:
*  Indication if particular configuration has been accepted or rejected.
*
*******************************************************************************/
uint8 CAN_RX_TxBufConfig(CAN_RX_TX_CFG *txConfig) 
{
    uint8 result = CAN_RX_FAIL;

    /* Write TX CMD Register */
    CY_SET_REG32((reg32 *) &CAN_RX_TX[txConfig->txmailbox].txcmd, (txConfig->txcmd |
                 CAN_RX_TX_WPN_SET));
    if ((CY_GET_REG32((reg32 *) & CAN_RX_TX[txConfig->txmailbox].txcmd) &
        CAN_RX_TX_READ_BACK_MASK) == (txConfig->txcmd & CAN_RX_TX_WPN_CLEAR))
    {
        /* Write TX ID Register */
        CY_SET_REG32( (reg32 *) &CAN_RX_TX[txConfig->txmailbox].txid, txConfig->txid);
        if (CY_GET_REG32( (reg32 *) &CAN_RX_TX[txConfig->txmailbox].txid) == txConfig->txid)
        {
            result = CYRET_SUCCESS;
        }
    }

    return(result);
}


/* [] END OF FILE */
