/*******************************************************************************
* File Name: CAN_RX_PM.c
* Version 1.50
*
* Description:
*  This file contains the setup, control and status commands to support 
*  component operations in low power mode.  
*
* Note:
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "CAN_RX.h"

static CAN_RX_BACKUP_STRUCT CAN_RX_backup = { 0u,
                                                                  0u,
                                                                  CAN_RX_INIT_INTERRUPT_MASK,
                                                                  CAN_RX_MODE_MASK };


/*******************************************************************************
* Function Name: CAN_RX_SaveConfig
********************************************************************************
*
* Summary:
*  Save CAN configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
* 
* Global Variables:
*  CAN_RX_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void CAN_RX_SaveConfig(void)
{
    CAN_RX_backup.intSr = (CY_GET_REG32(CAN_RX_INT_SR_PTR));
    CAN_RX_backup.intEn = (CY_GET_REG32(CAN_RX_INT_EN_PTR));
    CAN_RX_backup.cmd = (CY_GET_REG32(CAN_RX_CMD_PTR));
    CAN_RX_backup.cfg = (CY_GET_REG32(CAN_RX_CFG_PTR));
}


/*******************************************************************************
* Function Name: CAN_RX_RestoreConfig
********************************************************************************
*
* Summary:
*  Restore CAN configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  CAN_RX_backup - used when non-retention registers are restored.
*
* Side Effects:
*  If this API is called without first calling SaveConfig then in the following
*  registers will be default values from Customizer: CAN_RX_INT_SR, 
*  CAN_RX_INT_EN, CAN_RX_CMD, CAN_RX_CFG.
*
*******************************************************************************/
void CAN_RX_RestoreConfig(void) 
{
    CY_SET_REG32(CAN_RX_INT_SR_PTR, CAN_RX_backup.intSr);
    CY_SET_REG32(CAN_RX_INT_EN_PTR, CAN_RX_backup.intEn);
    CY_SET_REG32(CAN_RX_CMD_PTR, CAN_RX_backup.cmd);
    CY_SET_REG32(CAN_RX_CFG_PTR, CAN_RX_backup.cfg);
}


/*******************************************************************************
* Function Name: CAN_RX_Sleep
********************************************************************************
* 
* Summary:
*  Prepare CAN Component goes to sleep.
*
* Parameters:  
*  None.
*
* Return: 
*  None.
*
* Global Variables:
*  CAN_RX_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void CAN_RX_Sleep(void)
{   
    if((CAN_RX_CMD_REG.byte[0u] & CAN_RX_MODE_MASK) == CAN_RX_MODE_MASK)       
    {
        CAN_RX_backup.enableState = 1u;
    }
    else /* The CAN block is disabled */
    {
        CAN_RX_backup.enableState = 0u;
    }
    
    CAN_RX_SaveConfig();
    CAN_RX_Stop();    
}


/*******************************************************************************
* Function Name: CAN_RX_Wakeup
********************************************************************************
* 
* Summary:
*  Prepare CAN Component to wake up.
*
* Parameters:  
*  None.
*
* Return: 
*  None.
*
* Global Variables:
*  CAN_RX_backup - used when non-retention registers are restored.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void CAN_RX_Wakeup(void) 
{           
    /* Enable power to CAN */
    CAN_RX_PM_ACT_CFG_REG |= CAN_RX_ACT_PWR_EN;
    
    CAN_RX_RestoreConfig();
    
    if(CAN_RX_backup.enableState != 0u)
    {
        /* Enable component's operation */
        CAN_RX_Enable();
    } /* Do nothing if component's block was disabled before */
}


/* [] END OF FILE */
